from untwisted.network import core

def extern(obj, timeout=200):
    core.gear.timeout = 0

    def loop():
        obj.after(timeout, loop)               
        core.gear.update()
    
    obj.after(timeout, loop)               

def intern(obj, timeout):
    core.gear.timeout = timeout
    core.gear.pool.append(obj)










from traceback import print_exc as debug
from untwisted.core import Kill, Root
import inspect

class Stop(Exception):
    """
    This exception is used to avoid remaining handles being
    processed for a given event.

    from untwisted.dispatcher import Dispatcher, Stop
    
    def handle0(dispatcher):
        raise Stop
    
    def handle1(dispatcher):
        print 'it will not be processed!'
    
    dispatcher = Dispatcher()
    dispatcher.add_map('alpha', handle0)
    dispatcher.add_map('alpha', handle1)
    dispatcher.drive('alpha')
    """
    pass

class Erase(Exception):
    """
    When this exception is thrown from a handle it avoids such a handle
    being processed again upon its event.

    from untwisted.dispatcher import Dispatcher, Erase
    
    def handle(dispatcher):
        print 'It will be called just once!'
        raise Erase
    
    dispatcher = Dispatcher()
    dispatcher.add_map('alpha', handle)
    dispatcher.drive('alpha')
    dispatcher.drive('alpha')
    """

    pass

class Dispatcher(object):
    """
    The event dispatcher class.
    """

    base = dict()
    def __init__(self):
        self.base = dict()
        self.pool = list()

    def drive(self, event, *args):
        """
        Used to dispatch events.
        """

        try:
            self.process_base(event, args)
        except KeyError:
            pass

        try:
            self.process_static_base(event, args)
        except KeyError:
            pass

        for handle in self.pool:
            handle(self, event, args)

    def process_base(self, event, args):
        for handle, data in self.base[event][:]:
            try:
                seq = handle(self, *(args + data))
            except Stop:
                break
            except StopIteration:
                pass
            except Kill, Root:
                raise
            except Erase:
                self.base[event].remove((handle, data))
            except Exception as e:
                debug()
                self.drive(Exception, handle, e)

    def add_map(self, event, handle, *args):
        """
        Add a mapping like event -(arg0, arg1, arg2, ...)-> handle.
        """

        item = self.base.setdefault(event, list())
        item.append((handle, args))

    def del_map(self, event, handle, *args):
        """
        Remove a mapping like event -(arg0, arg1, arg2, ...)-> handle.
        """

        if args:
            self.base[event].remove((handle, args))
        else:
            self.base[event] = filter(lambda ind: 
                        ind[0] != handle, self.base[event])

    def install_maps(self, *args):
        """
        Install a set of mappings.
        """

        for ind in args:
            self.add_map(*ind)

    def insert_map(self, index, map):
        """

        """

        pass

    process_static_base = classmethod(process_base)
    add_static_map      = classmethod(add_map)
    del_static_map      = classmethod(del_map)

    def add_handle(self, handle):
        """
        Whenever an event occurs then handle is processed.
        """

        self.pool.append(handle)

    def del_handle(self, handle):
        """
        Avoid handle from being processed when a given event occurs.
        """

        self.pool.remove(handle)

xmap  = lambda dispatcher, *args: dispatcher.add_map(*args)
zmap  = lambda dispatcher, *args: dispatcher.del_map(*args)
spawn = lambda dispatcher, *args: dispatcher.drive(*args)

def once(spin, event, handle, *args):
    """
    Used to do a mapping like event -> handle
    but handle is called just once upon event.
    """

    def shell(*args):
        try:
            handle(*args)
        except Exception as e:
            pass
        finally:
            spin.del_map(event, handle, *args)
            raise e
    spin.add_map(event, handle, *args)









from untwisted.network import core, xmap, zmap, READ, WRITE, spawn
from untwisted.event import LOAD, get_event
import sys

class Fixed(object):
    """
    A handle for application layers that demand fixed chunk transmission of data.

    def on_found(spin, data):
        print data

    Fixed(spin, size=4)
    xmap(spin, FOUND, on_found)

    If it arises a chunk of size equal or greater than 4 then FOUND is processed.
    """

    FOUND = get_event()
    def __init__(self, spin, size=4):
        xmap(spin, LOAD, self.update)
        self.arr  = bytearray()
        self.size = size

    def update(self, spin, data):
        self.arr.extend(data)

        for ind in xrange(self.size, len(self.arr) + 1, self.size):
            chunk = buffer(self.arr, ind - self.size, self.size)
            spin.drive(Fixed.FOUND, chunk)
        else:
            try:
               del self.arr[:ind]
            except NameError:
                pass

class Breaker(object):
    """
    A handle for application layer protocols follows a command scheme pattern.

    def on_add(spin, *args):
        print args

    Breaker(spin)
    xmap(spin, 'add', on_add)

    If it arises the sequence of data defined below then on_add is processed and its
    arguments will be ('1', '2', '3').

        'add 1 2 3'

    
    """

    def __init__(self, device, delim=' '):
        self.delim = delim
        xmap(device, Terminator.FOUND, self.handle_found)

    def handle_found(self, device, data):
        lst = data.split(self.delim)
        spawn(device, lst.pop(0), *lst)

class Terminator:
    """
    A handle for application layer protocols that use '\r\n' as token.

    def on_found(spin, data):
        print data

    Terminator(spin, delim='\r\n')
    xmap(spin, FOUND, on_found)

    If it arises the sequence below then on_found is processed.

        'some-data\r\n'
    """

    FOUND = get_event()
    def __init__(self, spin, delim='\r\n'):
        self.delim  = delim
        self.arr    = bytearray()

        xmap(spin, LOAD, self.update)

    def update(self, spin, data):
        self.arr.extend(data)

        if not self.delim in data:
            return

        data = str(self.arr)
        lst  = data.split(self.delim)
        del self.arr[:]
        self.arr.extend(lst.pop(-1))

        for ind in lst:
            spin.drive(Terminator.FOUND, ind)
            
class Accumulator(object):
    """
    Just an accumulator on LOAD.
    """
    def __init__(self, spin):
        xmap(spin, LOAD, self.update)
        spin.accumulator  = self
        self.data = ''

    def update(self, spin, data):
        self.data = self.data + data

class AccUntil(object):
    DONE = get_event()
    def __init__(self, spin, data='', delim='\r\n\r\n'):
        self.delim = delim
        self.arr   = bytearray()
        self.spin  = spin
        xmap(spin, LOAD, self.update)
        self.update(spin, data)

    def update(self, spin, data):
        self.arr.extend(data)
        if self.delim in self.arr:
            self.process(spin)

    def process(self, spin):
        zmap(spin, LOAD, self.update)
        a, b = self.arr.split(self.delim, 1)
        spawn(spin, AccUntil.DONE, str(a), str(b))

class TmpFile(object):
    DONE = get_event()
    def __init__(self, spin, data, max_size, fd):
        self.fd       = fd
        self.max_size = max_size
        xmap(spin, LOAD, self.update)

        self.update(spin, data)

    def update(self, spin, data):
        pos   = self.fd.tell()
        count = self.max_size - pos
        self.fd.write(data[:count])

        if len(data) < count: 
            return

        zmap(spin, LOAD, self.update)
        spawn(spin, TmpFile.DONE, self.fd, data[count:])

# class AccAck(object):
    # DONE = get_event()
# 
    # def __init__(self, spin, fd, max_size, ack):
        # self.fd       = fd
        # self.max_size = size
        # xmap(spin, LOAD, self.update)
# 
    # def update(self, spin, data):
        # self.fd.write(data)
        # spin.dump(self.ack(fd, self.max_size))
# 
        # if self.fd.tell() >= self.max_size:
            # pass

def logcon(spin, fd=sys.stdout):
    def log(spin, data):
        fd.write('%s\n' % data)
    xmap(spin, Terminator.FOUND, log)







from untwisted.network import Spin, xmap, core
from untwisted.iostd import create_server, ACCEPT, LOAD

class EchoServer(object):
    def __init__(self, server):
        xmap(server, ACCEPT, lambda server, con: 
                     xmap(con, LOAD, lambda con, data: con.dump(data)))

if __name__ == '__main__':
    EchoServer(create_server('0.0.0.0', 1234, 5))
    core.gear.mainloop()





from socket import socket, AF_INET, SOCK_STREAM
from untwisted.network import Spin, xmap
from untwisted.event import READ
from threading import Lock

class Waker(object):
    MAX_SIZE = 6028
    def __init__(self):
        server = socket(AF_INET, SOCK_STREAM)
        server.bind(('127.0.0.1', 0))
        server.listen(1)

        client = Spin()
        client.connect_ex(server.getsockname())

        def consume(spin):
            spin.recv(self.MAX_SIZE)

        xmap(client, READ, consume)

        self.con, addr  = server.accept()
        self.lock       = Lock()

    def wake_up(self):
        with self.lock:
            self.con.send(' ')
    
waker = Waker()






from untwisted.network import core, SSL, xmap, die
from untwisted.iostd import put, lose
from untwisted.iossl import Client, Stdin, Stdout, SSL_CONNECT, SSL_CONNECT_ERR, SSL_CERTIFICATE_ERR, CLOSE, LOAD
from socket import socket, AF_INET, SOCK_STREAM
import ssl

def on_connect(con, host):
    Stdin(con)
    Stdout(con)
    con.dump("GET / HTTP/1.0\r\nHost: %s\r\n\r\n" % host)
    xmap(con, LOAD, put)
    xmap(con, CLOSE, lambda con, err: lose(con))

def main(addr, port, host):
    sock    = socket(AF_INET, SOCK_STREAM)
    context = ssl.create_default_context()
    con     = SSL(context.wrap_socket(sock, do_handshake_on_connect=False, server_hostname=host))
    con.connect_ex((addr, port))

    Client(con)
    xmap(con, SSL_CONNECT, on_connect, host)
    xmap(con, SSL_CONNECT_ERR, lambda con, err: die(err))
    xmap(con, SSL_CERTIFICATE_ERR, lambda con, err: die(err))

if __name__ == '__main__':
    from optparse import OptionParser

    parser   = OptionParser()
    parser.add_option("-a", "--addr", dest="addr",
                      metavar="string", default='localhost')
                  
    parser.add_option("-p", "--port", dest="port",
                      metavar="integer", default=443)

    parser.add_option("-o", "--host", dest="host",
                      metavar="string", default="www.google.com")

    (opt, args) = parser.parse_args()

    main(opt.addr, opt.port, opt.host)
    core.gear.mainloop()










from untwisted.network import Spin, spawn, xmap, zmap
from traceback import print_exc as debug
from untwisted.event import *
from collections import deque
import socket
import ssl

from errno import EALREADY, EINPROGRESS, EWOULDBLOCK, ECONNRESET, EINVAL, \
ENOTCONN, ESHUTDOWN, EINTR, EISCONN, EBADF, ECONNABORTED, EPIPE, EAGAIN 

CLOSE_ERR_CODE   = (ECONNRESET, ENOTCONN, ESHUTDOWN, ECONNABORTED, EPIPE, EBADF)
ACCEPT_ERR_CODE  = (EWOULDBLOCK, ECONNABORTED, EAGAIN)

class Dump(object):
    def process_error(self, spin, err):
        if err in CLOSE_ERR_CODE: 
            spawn(spin, CLOSE, err)
        else: 
            spawn(spin, SEND_ERR, err)

class DumpStr(Dump):
    __slots__ = 'data'

    def __init__(self, data=''):
        self.data = buffer(data)

    def process(self, spin):
        try:
            size = spin.send(self.data)  
        except socket.error as excpt:
            self.process_error(spin, excpt.args[0])
        else:
            self.data = buffer(self.data, size)

    def __nonzero__(self):
        return bool(self.data)

class DumpFile(DumpStr):
    __slots__ = 'fd'
    BLOCK     = 1024 * 124

    def __init__(self, fd):
        self.fd   = fd
        DumpStr.__init__(self)
        self.process_file()

    def process(self, spin):
        DumpStr.process(self, spin)
        if not self.data:
            self.process_file()

    def process_file(self):
        try:
            data = self.fd.read(DumpFile.BLOCK)
        except IOError as excpt:
            spawn(spin, READ_ERR, excpt)
        else:
            self.data = buffer(data)
        
class Stdin:
    """ 
    Stdin is a handle used to send data through Spin connections.

    Methods:
        dump     - Send data through the Spin instance.

        dumpfile - Dump a file through the Spin instance.

    Diagram:
        WRITE -> Stdin -(int:err, int:err, ())-> {**CLOSE, SEND_ERR, DUMPED}
    """

    def __init__(self, spin):
        """ 
        """

        self.queue    = deque()
        self.data     = None
        spin.dump     = self.dump
        spin.dumpfile = self.dumpfile
        self.spin     = spin

    def update(self, spin):
        """
        """
        
        if not self.data: 
            self.process_queue(spin)

        self.data.process(spin)

    def process_queue(self, spin):
        try:
            self.data = self.queue.popleft()
        except IndexError: 
            self.stop()

    def stop(self):
        zmap(self.spin, WRITE, self.update)
        spawn(self.spin, DUMPED)

    def start(self):
        if not self.queue: 
            xmap(self.spin, WRITE, self.update)

    def dump(self, data):
        """ 
        Send data through a Spin instance. 
        """

        self.start()
        dump = DumpStr(data)
        self.queue.append(dump)

    def dumpfile(self, fd):
        """ 
        Dump a file through a Spin instance. 
        """

        self.start()
        dump = DumpFile(fd)
        self.queue.append(dump)

class Stdout(object):
    """
    Used to read data through a Spin instance.

    Diagram:
    
        READ -> Stdout -(int:err, int:err, str:data)-> {**CLOSE, RECV_ERR, LOAD}
    """
    
    SIZE = 1024 * 124

    def __init__(self, spin):
        xmap(spin, READ, self.update)

    def update(self, spin):
        """
        """

        try:
            self.process_data(spin)
        except socket.error as excpt:
            self.process_error(spin, excpt.args[0])

    def process_data(self, spin):
        data = spin.recv(self.SIZE)

        if not data: 
            spawn(spin, CLOSE, '') 
        else: 
            spawn(spin, LOAD, data)

    def process_error(self, spin, err):
        if err in CLOSE_ERR_CODE: 
            spawn(spin, CLOSE, err)
        else: 
            spawn(spin, RECV_ERR, err)

class Client(object):
    """
    Used to set up TCP clients.

    Diagram:
    
        WRITE -> Client -((), int:err)-> {**CONNECT, **CONNECT_ERR}
    """

    def __init__(self, spin):
        xmap(spin, WRITE, self.update)

    def update(self, spin):
        zmap(spin, WRITE, self.update)
        err = spin.getsockopt(socket.SOL_SOCKET, socket.SO_ERROR)

        if err != 0:
            spawn(spin, CONNECT_ERR, err)
        else:
            spawn(spin, CONNECT)


class Server(object):
    """
    Used to set up TCP servers.

    READ -> Server -(Spin:client, int:err)-> {ACCEPT, ACCEPT_ERR}
    """

    def __init__(self, spin, wrap = lambda sock: Spin(sock)):
        xmap(spin, READ, self.update)
        self.wrap = wrap

    def update(self, spin):
        while True:
            try:
                sock, addr = spin.accept()
                spawn(spin, ACCEPT, self.wrap(sock))
            except socket.error as excpt:
                err = excpt.args[0]
                if not err in ACCEPT_ERR_CODE:
                    spawn(spin, ACCEPT_ERR, err)
                else:
                    break

def lose(spin):
    """
    It is used to close TCP connection and unregister
    the Spin instance from untwisted reactor.

    Diagram:

        lose -> (int:err | socket.error:err) -> CLOSE_ERR
    """

    try:
        spin.destroy()
    except Exception as err:
        spawn(spin, CLOSE_ERR, err)

    try:
        spin.close()
    except Exception as excpt:
        err = excpt.args[0]
        spawn(spin, CLOSE_ERR, err)

def put(spin, data):
    """
    A handle used to serialize arguments of events.
    
        xmap(con, LOAD, put)
    """
    print data

def create_server(addr, port, backlog):
    """
    Set up a TCP server and installs the basic handles Stdin, Stdout in the
    clients.

    Example:    

        def send_data(server, client):
            # No need to install Stdin or Stdout.
            client.dump('foo bar!')

        server = create_server('0.0.0.0', 1024, 50)
        xmap(server, on_accept, send_data)
    """

    server = Spin()
    server.bind(('', 1234))
    server.listen(200)
    Server(server)
    xmap(server, ACCEPT, lambda server, spin: install_basic_handles(spin))
    return server

def install_basic_handles(spin):
    """
    """

    Stdin(spin)
    Stdout(spin)
    xmap(spin, CLOSE, lambda spin, err: lose(spin))

def create_client(addr, port):
    """
    Set up a TCP client and installs the basic handles Stdin, Stdout.

    def send_data(client):
        client.dump('GET / HTTP/1.1\r\n')
        xmap(client, LOAD, iostd.put)

    client = create_client('www.google.com.br', 80)
    xmap(client, CONNECT, send_data)
    """

    spin = Spin()
    Client(spin)
    spin.connect_ex((addr, port))
    xmap(spin, CONNECT, install_basic_handles)
    return spin









# Whenever we use get_event it increases
# So we don't get events messed.
event_count = 0

def get_event():
    """ 
    It returns a new event signal.

    from untwisted.network import *
    X = get_event()
    Y = get_event()
    print X
    print Y


    Whenever we call get_event it increases the integer
    corresponding to the signal. That avoids events getting
    messed through modules.
    """

    global event_count

    event_count = event_count + 1
    return event_count

READ             = get_event()
WRITE            = get_event()
EXPT             = get_event()
ERROR            = get_event()
CLOSE            = get_event()
ACCEPT           = get_event()
CONNECT          = get_event()
CONNECT_ERR      = get_event()
LOAD             = get_event()
DUMPED           = get_event()
RECV_ERR         = get_event()
SEND_ERR         = get_event()
ACCEPT_ERR       = get_event()
READ_ERR         = get_event()
DUMPED_FILE      = get_event()
CLOSE_ERR        = get_event()
DONE             = get_event()
TIMEOUT          = get_event()
SSL_SEND_ERR        = get_event()
SSL_RECV_ERR        = get_event()
SSL_CERTIFICATE_ERR = get_event()
SSL_CONNECT_ERR     = get_event()
SSL_CONNECT         = get_event()










from untwisted.network import core, xmap
from untwisted.job import Job, DONE
import time

def sum(x, y):
    time.sleep(3)
    return x + y

def show(job, result):
    print result

for ind in xrange(100):
    job = Job(sum, ind, 1000)
    xmap(job, DONE, show)

core.gear.mainloop()






from untwisted.network import xmap, zmap, spawn, Erase
from untwisted import iostd
from untwisted.event import get_event
from untwisted.event import WRITE, READ, CONNECT, CONNECT_ERR, CLOSE,      \
SEND_ERR, RECV_ERR, LOAD, SSL_SEND_ERR, SSL_RECV_ERR, SSL_CERTIFICATE_ERR, \
SSL_CONNECT_ERR, SSL_CONNECT
import socket
import ssl

class DumpStr(iostd.DumpStr):
    def process(self, spin):
        try:
            size = spin.send(self.data)  
        except ssl.SSLError as excpt:
            spawn(spin, SSL_SEND_ERR, excpt)
        except socket.error as excpt:
            self.process_error(spin, excpt.args[0])
        else:
            self.data = buffer(self.data, size)

class DumpFile(DumpStr, iostd.DumpFile):
    pass

class Stdin(iostd.Stdin):
    def dump(self, data):
        self.start()
        data = DumpStr(data)
        self.queue.append(data)

    def dumpfile(self, fd):
        self.start()
        fd = DumpFile(fd)
        self.queue.append(fd)

class Stdout(iostd.Stdout):
    def update(self, spin):
        try:
            while True:
                self.process_data(spin)
        except ssl.SSLError as excpt:
            spawn(spin, SSL_RECV_ERR, spin, excpt)
        except socket.error as excpt:
            self.process_error(spin, excpt.args[0])

class Handshake(object):
    """ 
    """

    def __init__(self, spin):
        xmap(spin, WRITE, self.do_handshake)

    def do_handshake(self, spin):
        """
        """

        try:
            spin.do_handshake()
        except ssl.CertificateError as excpt:
            spawn(spin, SSL_CERTIFICATE_ERR, excpt)
        except ssl.SSLWantReadError:
            pass
        except ssl.SSLWantWriteError:
            pass
        except ssl.SSLError as excpt:
            spawn(spin, SSL_CONNECT_ERR, excpt)
        else:
            spawn(spin, SSL_CONNECT)
            raise Erase

class Client(iostd.Client):
    def update(self, spin):
        iostd.Client.update(self, spin)
        Handshake(spin)

class Server(iostd.Server):
    def __init__(self, spin):
        xmap(spin, READ, self.update)

    def update(self, spin):
        pass

def create_ssl_server():
    pass

def create_ssl_client(addr, port):
    pass








""" 
"""

from untwisted.network import xmap, zmap, core, spawn
from untwisted.iostd import Stdin, Stdout, Server, DUMPED, lose, LOAD, ACCEPT, CLOSE
from untwisted.splits import AccUntil, TmpFile
from untwisted.timer import Timer
from untwisted.event import get_event
from untwisted.debug import on_event, on_all
from untwisted import network

from urlparse import parse_qs
from cgi import FieldStorage
from tempfile import TemporaryFile as tmpfile

from socket import socket, AF_INET, SOCK_STREAM, SOL_SOCKET, SO_REUSEADDR
from os.path import getsize
from mimetypes import guess_type
from os.path import isfile, join, abspath, basename, dirname
from jinja2 import Template, FileSystemLoader, Environment
import argparse

class Headers(dict):
    def __init__(self, data):
        for ind in data:
            field, sep, value = ind.partition(':')
            self[field.lower()] = value

class Spin(network.Spin):
    def __init__(self, sock, app):
        network.Spin.__init__(self, sock)
        self.app          = app
        self.response     = ''
        self.headers      = dict()
        self.data         = ''
        self.add_default_headers()

    def add_default_headers(self):
        self.set_response('HTTP/1.1 200 OK')
        self.add_header(('Server', 'Rapidserv'))

    def set_response(self, data):
        self.response = data

    def add_header(self, *args):
        for key, value in args:
            self.headers[str(key).lower()] = str(value)

    def add_data(self, data, mimetype='text/html;charset=utf-8'):
        self.add_header(('Content-Type', mimetype))
        self.data = str(data)

    def done(self):
        self.headers['Content-Length'] = len(self.data)
        self.send_headers()
        self.dump(self.data)
        self.data = ''
        self.headers.clear()
        self.add_default_headers()

    def send_headers(self):
        """
        """

        data = self.response
        for key, value in self.headers.iteritems():
            data = data + '\r\n' + '%s:%s' % (key, value)
        data = data + '\r\n\r\n'
        self.dump(data)

    def render(self, template_name, *args, **kwargs):
        template = self.app.env.get_template(template_name)
        self.add_data(template.render(*args, **kwargs))

class RapidServ(object):
    """
    """

    def __init__(self, app_dir, static_dir='static', template_dir='templates', auto_reload=True):
        self.app_dir      = dirname(abspath(app_dir))
        self.static_dir   = static_dir
        self.template_dir = template_dir
        self.loader       = FileSystemLoader(searchpath = join(self.app_dir, self.template_dir))
        self.env          = Environment(loader=self.loader, auto_reload=auto_reload)
        sock              = socket(AF_INET, SOCK_STREAM)
        self.local        = network.Spin(sock)
        self.local.setsockopt(SOL_SOCKET, SO_REUSEADDR, 1)


    def bind(self, addr, port, backlog):
        Server(self.local, lambda sock: Spin(sock, self)) 
        self.local.bind((addr, port))
        self.local.listen(backlog)
        
        xmap(self.local, ACCEPT, self.handle_accept)

    def run(self):
        parser = argparse.ArgumentParser()
        parser.add_argument('-a', '--addr',  default='0.0.0.0', help='Address')
        parser.add_argument('-p', '--port', type=int, default=80, help='Port')
        parser.add_argument('-b', '--backlog',  type=int, default=50, help='Port')
        args = parser.parse_args()

        self.bind(args.addr, args.port, args.backlog)
        core.gear.mainloop()

    def handle_accept(self, local, spin):
        Stdin(spin)
        Stdout(spin)
        AccUntil(spin)
        HttpTransferHandle(spin)
        HttpRequestHandle(spin)
        HttpMethodHandle(spin)

        # must be improved.
        Locate(spin)

        # NonPersistentConnection(spin)

        PersistentConnection(spin)
        # InvalidRequest(client)

        xmap(spin, CLOSE, lambda con, err: lose(con))

    def route(self, method):
        """
        """

        def shell(handle):
            xmap(self.local, ACCEPT, lambda local, spin: 
                 xmap(spin, method, self.build_kw, handle))
            return handle
        return shell

    def request(self, method):
        """
        """

        def shell(handle):
            xmap(self.local, ACCEPT, lambda local, spin: 
                 xmap(spin, method, handle))
            return handle
        return shell

    def build_kw(self, spin, request, handle):
        """
        """

        kwargs = dict()
        kwargs.update(request.query)

        if request.data: 
            kwargs.update(request.data)
        handle(spin, **kwargs)

    def accept(self, handle):
        xmap(self.local, ACCEPT, lambda local, spin: handle(spin))

    def overflow(self, handle):
        xmap(self.local, ACCEPT, lambda local, spin: 
                    xmap(spin, HttpRequestHandle.OVERFLOW, handle))

class Request(object):
    def __init__(self, data):
        headers                              = data.split('\r\n')
        request                              = headers.pop(0)
        self.method, self.path, self.version = request.split(' ')
        self.headers                         = Headers(headers)
        self.fd                              = tmpfile('a+')
        self.data                            = None
        self.path, sep, self.query           = self.path.partition('?')
        self.query                           = parse_qs(self.query)

    def build_data(self):
        self.fd.seek(0)
        self.data = FieldStorage(fp=self.fd, environ=get_env(self.headers))

class HttpTransferHandle(object):
    HTTP_TRANSFER = get_event()

    def __init__(self, spin):
        xmap(spin, AccUntil.DONE, self.process_request)

    def process_request(self, spin, request, data):
        request = Request(request)
        spawn(spin, HttpTransferHandle.HTTP_TRANSFER, request, data)

class HttpRequestHandle(object):
    HTTP_REQUEST = get_event()
    OVERFLOW     = get_event()
    MAX_SIZE     = 1024 * 5024

    def __init__(self, spin):
        self.request = None
        xmap(spin, HttpTransferHandle.HTTP_TRANSFER, self.process)
        xmap(spin, TmpFile.DONE,  
                   lambda spin, fd, data: spawn(spin, 
                                 HttpRequestHandle.HTTP_REQUEST, self.request))

    def process(self, spin, request, data):
        size         = int(request.headers.get('content-length', '0'))
        self.request = request

        if HttpRequestHandle.MAX_SIZE <= size:
            spawn(spin, HttpRequestHandle.OVERFLOW, request)
        else:
            TmpFile(spin, data, size, request.fd)

class HttpMethodHandle(object):
    def __init__(self, spin):
        xmap(spin, HttpRequestHandle.HTTP_REQUEST, self.process)

    def process(self, spin, request):
        request.build_data()
        spawn(spin, request.method, request)
        spawn(spin, '%s %s' % (request.method, request.path), request)
        spin.dump('')

class NonPersistentConnection(object):
    def __init__(self, spin):
        xmap(spin, DUMPED, lambda con: lose(con))

class PersistentConnection(object):
    TIMEOUT = 10
    MAX     = 10

    def __init__(self, spin):
        self.timer = Timer(PersistentConnection.TIMEOUT, lambda: lose(spin))
        self.count = 0
        xmap(spin, TmpFile.DONE, self.process)
        xmap(spin, DUMPED, self.install_timeout)
        xmap(spin, HttpTransferHandle.HTTP_TRANSFER, lambda spin, request, data: self.timer.cancel())

        spin.add_header(('connection', 'keep-alive'))
        spin.add_header(('keep-alive', 'timeout=%s, max=%s' % (PersistentConnection.TIMEOUT, 
                                                                        PersistentConnection.MAX)))

    def process(self, spin, fd, data):
        self.count = self.count + 1

        if self.count < PersistentConnection.MAX:
            AccUntil(spin, data)

    def install_timeout(self, spin):
        self.timer = Timer(PersistentConnection.TIMEOUT, lambda: lose(spin))

class DebugRequest(object):
    def __init__(self, spin):
        xmap(spin, HttpRequestHandle.HTTP_REQUEST, self.process)

    def process(self, spin, request):
        print request.method
        print request.path
        print request.data
        print request.headers

class InvalidRequest(object):
    """ 
    """

    def __init__(self, spin):
        # xmap(spin, INVALID_BODY_SIZE, self.error)
        # xmap(spin, IDLE_TIMEOUT, self.error)
        pass

    def error(self, spin):
        spin.set_response('HTTP/1.1 400 Bad request')
        HTML = '<html> <body> <h1> Bad request </h1> </body> </html>'
        spin.add_data(HTML)
        spin.done()

class Locate(object):
    """
    """

    def __init__(self, spin):
        xmap(spin, 'GET', self.locate)

    def locate(self, spin, request):
        path = join(spin.app.app_dir, spin.app.static_dir, basename(request.path))
        if not isfile(path):
            return

        # Where we are going to serve files.
        # I might spawn an event like FILE_NOT_FOUND.
        # So, users could use it to send appropriate answers.
        type_file, encoding = guess_type(path)
        default_type = 'application/octet-stream'

        spin.add_header(('Content-Type', type_file if type_file else default_type),
                     ('Content-Length', getsize(path)))

        spin.send_headers()
        xmap(spin, OPEN_FILE_ERR, lambda con, err: lose(con))
        drop(spin, path)

def get_env(header):
    """
    Shouldn't be called outside this module.
    """

    environ = {
                'REQUEST_METHOD':'POST',
                'CONTENT_LENGTH':header.get('content-length', 0),
                'CONTENT_TYPE':header.get('content-type', 'text')
              }

    return environ


OPEN_FILE_ERR = get_event()
def drop(spin, filename):
    """
    Shouldn't be called outside this module.
    """

    try:
        fd = open(filename, 'rb')             
    except IOError as excpt:
        err = excpt.args[0]
        spawn(spin, OPEN_FILE_ERR, err)
    else:
        spin.dumpfile(fd)

def make(searchpath, folder):
    """
    Used to build a path search for Locate plugin.
    """

    from os.path import join, abspath, dirname
    searchpath = join(dirname(abspath(searchpath)), folder)
    return searchpath












from untwisted.plugins.requests import get, HttpResponseHandle
from untwisted.network import xmap, core

def on_done(con, response):
    print response.headers
    print response.code
    print response.version
    print response.reason 
    print response.fd.read()

if __name__ == '__main__':
    urls = ['www.bol.uol.com.br', 'www.google.com']
    
    for ind in urls:
        con = get(ind, 80, '/')
        xmap(con, HttpResponseHandle.HTTP_RESPONSE, on_done)
    core.gear.mainloop()








def ip_to_long (ip):
    """
    Convert ip address to a network byte order 32-bit integer.
   """
    quad = ip.split('.')
    if len(quad) == 1:
        quad = quad + [0, 0, 0]
    elif len(quad) < 4:
        host = quad[-1:]
        quad = quad[:-1] + [0,] * (4 - len(quad)) + host

    lip = 0
    for q in quad:
        lip = (lip << 8) | int(q)
    return lip


def long_to_ip (l):
    """
    Convert 32-bit integerto to a ip address.
    """
    return '%d.%d.%d.%d' % (l>>24 & 255, l>>16 & 255, l>>8 & 255, l & 255) 






# It imports basic untwisted objects.
from untwisted.network import *
from untwisted.iostd import *
from untwisted.splits import Terminator, logcon
# The untwisted schedule event.
from untwisted.timer import Sched
from untwisted.plugins.irc import *
from socket import socket
# We need it to delay when sending irc commands
# otherwise we can go down by excess of flood.
from time import sleep


def send_auth(con, nick, user, cmd, delay):
    Stdin(con)
    Stdout(con)
    Terminator(con)
    Irc(con)
    logcon(con)

    def do_job(spin, *args):
        for ind in cmd:
            send_cmd(spin, ind)
            sleep(delay)
    
    xmap(con, '376', do_job)
    xmap(con, 'PING', lambda con, prefix, 
            servaddr: send_cmd(con, 'PONG :%s' % servaddr))

    xmap(con, CLOSE, lambda con, err: lose(con))
    send_cmd(con, 'NICK %s' % nick)
    send_cmd(con, 'USER %s' % user)

def main(address, port, nick, user, cmd, delay=1):
    sock = socket(AF_INET, SOCK_STREAM)
    con  = Spin(sock)
    
    Client(con)
    con.connect_ex((address, port))
    xmap(con, CONNECT, send_auth, nick, user, cmd, delay)
    return con

if __name__ == '__main__':
    USER     = 'uxirc beta gama :uxirc'
    NICK     = 'alpha'
    CMD      = ('JOIN #vy','PRIVMSG #vy :Uriel', 'quit')
    INTERVAL = 10
    cbck     = lambda :main('irc.freenode.com', 6667, NICK, USER, CMD)
    Sched(INTERVAL, cbck)

    core.gear.mainloop()







"""
"""

from untwisted.plugins.rapidserv import RapidServ, core, xmap

app = RapidServ(__file__)

@app.accept
class Simple(object):
    def __init__(self, con):
        xmap(con, 'GET /', self.send_base)

    def send_base(self, con, request):
        HTML = """ <html> 
                   <body>
                   <p> It is simple :P </p>
                   </body> </html>
               """

        con.add_data(HTML)
        con.done()

if __name__ == '__main__':
    app.bind('0.0.0.0', 80, 60)
    core.gear.mainloop()













"""
"""

from untwisted.plugins.rapidserv import RapidServ, make
import sqlite3

DB_FILENAME = 'DB'
DB          = sqlite3.connect(make(__file__, DB_FILENAME))
app         = RapidServ(__file__)
DB.execute('CREATE TABLE IF NOT EXISTS quotes (id  INTEGER PRIMARY KEY, name TEXT, quote TEXT)')
DB.commit()

@app.request('GET /')
def send_base(con, request):
    rst = DB.execute('SELECT * FROM quotes')
    con.render('show.jinja', posts = rst.fetchall())
    con.done()

@app.request('GET /load_index')
def load_index(con, request):
    index        = request.query['index']
    rst          = DB.execute('SELECT name, quote FROM quotes where id=?', index)
    name, quote  = rst.fetchone()
    con.render('view.jinja', name=name, quote=quote)
    con.done()

@app.request('GET /add_quote')
def add_quote(con, request):
    name      = request.query['name'][0]
    quote     = request.query['quote'][0]
    DB.execute("INSERT INTO quotes (name, quote) VALUES %s" % repr((name, quote)))
    DB.commit()
    send_base(con, request)

if __name__ == '__main__':
    app.run()
    





# It imports basic untwisted objects.
# The irc protocol implementation.
from ircjob import xmap, zmap
import ircjob

def on_join(spin, *args):
    print 'on_join\n', args

def on_part(spin, *args):
    print 'on_part\n', args

def on_privmsg(spin, *args):
    print 'on_privmsg\n', args

def on_332(spin, *args):
    print 'on_332\n', args

def on_302(spin, *args):
    print 'on_302\n', args

def on_333(spin, *args):
    print 'on_333\n', args

def on_353(spin, *args):
    print 'on_353\n', args

def on_366(spin, *args):
    print 'on_366\n', args

def on_474(spin, *args):
    print 'on_474\n', args

def on_302(spin, *args):
    print 'on_302\n', args

def on_376(spin, *args):
    print 'on_376\n', args

def on_notice(spin, *args):
    print 'on_notice\n', args

def on_001(spin, *args):
    print 'on_001\n', args

def on_002(spin, *args):
    print 'on_002\n', args

def on_003(spin, *args):
    print 'on_003\n', args

def on_004(spin, *args):
    print 'on_004\n', args

if __name__ == '__main__':
    USER = 'uxirc uxirc uxirc :uxirc'
    NICK = 'uxirc'
    CMD = ('JOIN #&math',
           'PRIVMSG #&math :hello',
           'topic #calculus',
           'PART #&math :bye',
           'userhost %s' % NICK)
    
    # This call back will be called periodically.
    con = ircjob.main('irc.freenode.org', 6667, NICK, USER, CMD)

    xmap(con, 'JOIN', on_join)
    xmap(con, 'PART', on_part)
    xmap(con, '376', on_376)
    xmap(con, 'NOTICE', on_notice)
    xmap(con, 'PRIVMSG', on_privmsg)
    xmap(con, '332', on_332)
    xmap(con, '001', on_001)
    xmap(con, '001', on_002)
    xmap(con, '003', on_003)
    xmap(con, '004', on_004)
    xmap(con, '333', on_333)
    xmap(con, '353', on_353)
    xmap(con, '366', on_366)
    xmap(con, '474', on_474)
    xmap(con, '302', on_302)

    # it runs the reactor.
    ircjob.gear.mainloop()




""" deb_5.py 
    Usage: 
"""
from untwisted.network import *
from untwisted.utils.stdio import *
from untwisted.utils.shrug import *
from untwisted.task import Task, COMPLETE

import sys
import random
MAX_CON = 200
LOW = 0
HIGH = 5

PORT = 1234

def on_accept(serv, con):
    Stdin(con)
    Stdout(con)
    
    cookie = random.randint(LOW, HIGH)
    #print cookie
    con.dump('%s\r\n' % cookie)

    xmap(con, DUMPED, lose)

def set_up_server():
    sock = socket(AF_INET, SOCK_STREAM)
    serv = Spin(sock)
    
    serv.bind(('', PORT))
    serv.listen(300)
    Server(serv)

    xmap(serv, ACCEPT, on_accept)

def done(task, data):
    print data
    raise Kill

def on_connect(con):
    Stdin(con)
    Stdout(con)
    Shrug(con)
    xmap(con, FOUND, lambda con, data: spawn(con, data, data))
    #xmap(con, FOUND, lambda con, data: sys.stdout.write('%s\n' % data))
    print 'connected' 

def main():
    task = Task(dict())
    set_up_server()

  
    for ind in xrange(MAX_CON):
        sock = socket(AF_INET, SOCK_STREAM)
        con = Spin(sock)
        Client(con)

        xmap(con, CONNECT, on_connect)
        xmap(con, CONNECT_ERR, on_connect_err)
        con.connect_ex(('localhost', PORT))

        def job(data, event, args, index=ind):
            base = data.setdefault(event, list())
            base.append(('Con %s' % index, args[1]))
            return True

        task.gather(con, 
                         ('0', job),
                         ('1', job),
                         ('2', job),
                         ('3', job),
                         ('4', job),
                         ('5', job),
                         (CONNECT_ERR, lambda *args: True)) 



    xmap(task, COMPLETE, done)

def on_connect_err(con, err):
    print 'on_connect_err %s.' % err

if __name__ == '__main__':
    main()
    #install_reactor(core.Select)
    core.gear.mainloop()




"""
"""

from untwisted.network import core, Spin, xmap
from untwisted.iostd import create_server, ACCEPT, CLOSE, lose
from untwisted.splits import Terminator
from untwisted.tools import coroutine

class ChatServer(object):
    def __init__(self, server):
        xmap(server, ACCEPT, self.handle_accept)
        self.pool = []

    @coroutine
    def handle_accept(self, server, client):
        Terminator(client, delim='\r\n')
        xmap(client, CLOSE, lambda client, err: self.pool.remove(client))

        client.dump('Type a nick.\r\nNick:')    
        client.nick, = yield client, Terminator.FOUND

        xmap(client, Terminator.FOUND, self.echo_msg)
        self.pool.append(client)

    def echo_msg(self, client, data):
        for ind in self.pool:
            if not ind is client:
                ind.dump('%s:%s\r\n' % (client.nick, data))

if __name__ == '__main__':
    server = create_server('', 1234, 5)
    ChatServer(server)
    core.gear.mainloop()






from untwisted.plugins.rapidserv import RapidServ, core

app = RapidServ(__file__)

@app.request('GET /')
def send_base(con, request):
    con.add_data('<html> <body> <p> Rapidserv </p> </body> </html>')
    con.done()

if __name__ == '__main__':
    app.bind('0.0.0.0', 8000, 50)
    core.gear.mainloop()






"""
    Name: deb_2.py
    Description: It is a unit test for DccClient, DccServer classes.
    It creates DccServer instances for sending files
    and DccClient instances for receiving files
    it checkes whether the data was sent correctly
    through the localhost interface.
"""

from untwisted.network import *
from untwisted.event import *
from untwisted.plugins.irc import *
from StringIO import StringIO
import sys
logvar = sys.stdout

def check(client, fd, data, ident):
    fd.seek(0)
    chunk = fd.read() 
    if chunk == data:
        logvar.write('Passed %s\n' % ident)
    else:
        logvar.write('Failed %s\n' % ident)

def run_test(n, m, data):
    size = len(data)
     
    for ind in xrange(n, m):
        # It creates a copy of data
        # that is wrapped with StringIO
        # so it behaves like a file.
        fd = StringIO(data) 
        # It is like receiving a file on a port
        # whose value is ind.
        DccServer(fd, ind)
    
    for ind in xrange(n, m):
        fd = StringIO()
        # It tries to connect to the port.
        client = DccClient('localhost', ind, fd, size)
        # When DONE is spawned we check for data corruption.
        xmap(client,  DONE, check, fd, data, ind)

def main():
    # It is like sending 1400 - 1234 files of 4 * 100000 size
    # and receiving the same amount of data.
    # If it when DONE is issued it checkes
    # if it received exactly what it sent.
    run_test(1234, 1400, 'abcd' * 100000)
    gear.mainloop()

main()




from untwisted.network import xmap, Spin, core
from untwisted.iostd import Client, Stdout, Stdin, CONNECT, DUMPED
from socket import socket, AF_INET, SOCK_STREAM
from untwisted.core import die

def setup(con, msg):
    Stdout(con)
    Stdin(con)
    con.dump(msg)
    xmap(con, DUMPED, lambda con: die('Msg dumped!'))

def create_connection(addr, port, msg):
    sock = socket(AF_INET, SOCK_STREAM)
    con  = Spin(sock)

    Client(con)
    con.connect_ex((addr, port))
    xmap(con, CONNECT, setup, msg)

if __name__ == '__main__':
    from optparse import OptionParser

    parser   = OptionParser()
    parser.add_option("-a", "--addr", dest="addr",
                      metavar="string", default='localhost')
                  
    parser.add_option("-p", "--port", dest="port",
                      type="int", default=1234)

    parser.add_option("-m", "--msg", dest="msg",
                      metavar="string")

    (opt, args) = parser.parse_args()
    create_connection(opt.addr, opt.port, opt.msg)
    core.gear.mainloop()







from untwisted.network import core, xmap, Device
from untwisted.iofile import *
from subprocess import Popen, PIPE, STDOUT
from untwisted.core import Kill

def on_close(dev, err):
    print 'On CLOSE ...', err
    lose(dev)
    raise Kill

def on_load(dev, data):
    print 'On LOAD ...', data

child   = Popen(['python2', '-i'], stdout=PIPE, stdin=PIPE, stderr=STDOUT)

stdout  = Device(child.stdout)
stdin   = Device(child.stdin)

Stdin(stdin)
Stdout(stdout)

stdin.dump('print "hello world!"\n')
xmap(stdin, DUMPED, lambda dev: stdin.dump('quit()\n'))
xmap(stdout, LOAD, on_load)
xmap(stdin, CLOSE, on_close)
xmap(stdout, CLOSE, on_close)

core.gear.mainloop()





from untwisted.network import xmap, spawn
from untwisted import iostd
from untwisted.event import CLOSE, LOAD, DUMPED
import os

class DumpStr(iostd.DumpStr):
    def process(self, dev):
        try:
            size = os.write(dev.fd, self.data)  
        except OSError as excpt:
            self.process_error(dev, excpt.args[0])
        else:
            self.data = buffer(self.data, size)

class DumpFile(DumpStr, iostd.DumpFile):
    pass

class Stdin(iostd.Stdin):
    """
    Used to dump data through a Device instance.

    Methods:
        dump     - Send data through a Device instance.

        dumpfile - Dump a file through a Device instance.

    Diagram:
        WRITE -> Stdin -(int:err, ())-> {**CLOSE, DUMPED}

    """

    def dump(self, data):
        self.start()
        data = DumpStr(data)
        self.queue.append(data)

    def dumpfile(self, fd):
        self.start()
        fd = DumpFile(fd)
        self.queue.append(fd)

class Stdout(iostd.Stdout):
    """
    Used to read data from a Device instance.

    Diagram:
    
        READ -> Stdout -(int:err, int:err, str:data)-> {**CLOSE, RECV_ERR, LOAD}
    """

    def update(self, dev):
        try:
            self.process_data(dev)
        except OSError as excpt:
            self.process_error(dev, excpt.args[0])

    def process_data(self, dev):
        data = os.read(dev.fd, self.SIZE)
        if not data: 
            spawn(dev, CLOSE, '') 
        else: 
            spawn(dev, LOAD, data)

def lose(device):
    """
    It is used to close the device and destroy the Device instance.
    """
    device.destroy()
    device.close()









from untwisted.event import READ, WRITE, ERROR, EXPT
from select import *
from socket import *

class Root(Exception):
    pass

class Kill(Exception):
    pass

def die(msg=''):
    """
    It is used to stop the reactor.
    """

    print msg
    raise Kill

class Gear(object):
    """ 
    It implements a basic set of methods that should be
    common to all reactors.

    This class isn't intented to be subclassed outside untwisted
    modules.

    I thought of installing the reactor inside this class
    instead of inside this module. However, it arose some
    odd patterns which i preferred to avoid. 

    """

    MAX_SIZE = 6028
    def __init__(self):
        """ Class constructor """
        self.pool = []

    def mainloop(self):
        """ 
        This is the reactor mainloop.
        It is intented to be called when
        a reactor is installed.

        from untwisted.network import *

        # It processes forever.
        core.gear.mainloop()
        """
            
        while True:
        # It calls repeteadly the reactor
        # update method.
            try:
                self.update()
            except Kill:
        # It breaks the loop
        # silently.
        # people implementing reactors from other mainloop
        # should implement this try: catch
        # suitably to their needs.

                break

    def process_pool(self):
        """ 
        This method processes the pool of objects
        that are binded to the reactor. 

        This method shouldn't be called by the
        user of the class except he knows what he is
        doing.
        """
        for ind in self.pool:
            ind.update()

    def update(self):
        """
        This method is intented to be subclassed by
        reactor classes.
        """

    def register(self, spin):
        pass

    def unregister(self, spin):
        pass

        pass


class Select(Gear):
    """
    This reactor is select based. It is default on non posix platforms.
    """

    def __init__(self):
        # This variable holds the timeout passed
        # to select.
        self.timeout = None

        # This is the default timeout. It is used
        # by objects being processed in pool.
        self.default_timeout = None

        self.base  = []

        # These are the sockets in R/W status.
        self.rsock = set()
        self.wsock = set()
        self.xsock = set()
        Gear.__init__(self)

    def update(self):
        """ 
        Other reactors should call this method.
        """

        self.process_pool()
        self.process_reactor()

    def process_reactor(self):
        rsock, wsock, xsock = select(self.rsock , self.wsock , 
                                            self.xsock, self.timeout)

        for ind in wsock:
            try:
                ind.drive(WRITE)
            except Root:
                pass

        for ind in rsock:
            try:
                ind.drive(READ)
            except Root:
                pass

    def register(self, spin):
        self.base.append(spin)

    def unregister(self, spin):
        self.base.remove(spin)
        self.del_rsock(spin)
        self.del_wsock(spin)

    def update_rsock(self, spin):
        """
        """

        if spin.base.get(READ):
            self.add_rsock(spin)
        else:
            self.del_rsock(spin)

    def scale(self, spin):
        """
        """

        self.update_wsock(spin)
        self.update_rsock(spin)

    def update_wsock(self, spin):
        """
        """

        if spin.base.get(WRITE):
            self.add_wsock(spin)
        else:
            self.del_wsock(spin)

    def add_rsock(self, spin):
        """
        """

        self.rsock.add(spin)

    def add_wsock(self, spin):
        """
        """

        self.wsock.add(spin)

    def del_wsock(self, spin):
        """
        """

        try:
            self.wsock.remove(spin)
        except KeyError:
            pass

    def del_rsock(self, spin):
        """
        """

        try:
            self.rsock.remove(spin)
        except KeyError:
            pass


class Epoll(Gear):
    """
    This reactor is epoll based. It is default on posix platforms.
    """

    def __init__(self):
        Gear.__init__(self)
        # epoll uses -1 as default for timeout.
        self.timeout         = -1
        self.default_timeout = -1
        self.base            = dict()
        self.pollster        = epoll()


    def update(self):
        """
        Other reactors should call this method.
        """

        self.process_pool()
        self.process_reactor()

    def process_reactor(self):
        """
        """

        events = self.pollster.poll(self.timeout) 
        for fd, event in events:
            try:
                self.dispatch(fd, event)
            except KeyError:
                pass

    def register(self, spin):
        """
        """

        self.base[spin.fd] = spin
        self.pollster.register(spin.fd)

    def unregister(self, spin):
        """
        """

        del self.base[spin.fd]
        self.pollster.unregister(spin.fd)

    def scale(self, spin):
        """
        """

        is_readable = EPOLLIN  if spin.base.get(READ) else 0 
        is_writable = EPOLLOUT if spin.base.get(WRITE) else 0
        mask        = is_readable | is_writable
        self.pollster.modify(spin.fd, mask)

    def dispatch(self, fd, event):
        """
        """

        spin = self.base[fd]

        if event & EPOLLOUT:
            try:
                spin.drive(WRITE)
            except Root:
                pass

        if event & EPOLLIN:
            try:
                spin.drive(READ)
            except Root:
                pass

class Poll(object):
    def __init__(self, timeout=None):
        self.timeout = timeout
        self.base    = []
       
def install_reactor(reactor, *args, **kwargs):
    global gear
    gear = reactor(*args, **kwargs)
    #print 'Using %s reactor' % reactor

def default():
    try:
        install_reactor(Epoll)
    except NameError:
        install_reactor(Select)

default()
# install_reactor(Select)
# install_reactor(Epoll)

# __all__ = ['get_event', 'READ', 'WRITE' , 'install_reactor']
















"""
"""

from untwisted.network import core, Spin, xmap
from untwisted.iostd import *
from untwisted.splits import Terminator

class IrcHub(object):
    def __init__(self, server_port, backlog, irc_address, irc_port):
        self.irc_address = irc_address
        self.irc_port    = irc_port

        server = Spin()
        server.bind(('', server_port))
        server.listen(int(backlog))
    
        Server(server)
    
        xmap(server, ACCEPT, self.handle_accept)

    def handle_accept(self, server, client):
        Stdin(client)
        Stdout(client)
        Terminator(client, delim='\r\n')

        xmap(client, Terminator.FOUND, self.handle_found)
        xmap(client, CLOSE, self.down_connection)

        irc = Spin()
        Client(irc)
        Stdin(irc)
        Stdout(irc)
        Shrug(irc, delim='\r\n')
        xmap(irc, Terminator.FOUND, self.handle_found)
        xmap(irc, CONNECT, self.handle_connect)
        xmap(irc, CONNECT_ERR, self.down_connection)
        xmap(irc, CLOSE, self.down_connection)

        irc.arrow    = client
        client.arrow = irc
        
        irc.connect_ex((self.irc_address, self.irc_port))

    def handle_connect(self, irc):
        print 'Connection to %s:%s stablished.' % irc.getpeername()


    def down_connection(self, transport, err):
        transport.arrow.close()
        transport.arrow.destroy()
        transport.close()
        transport.destroy()

    def handle_found(self, transport, data):
        transport.arrow.dump('%s\r\n' % data)
        print data

if __name__ == '__main__':
    import sys

    script, server_port, backlog, target_address, target_port = sys.argv
    IrcHub(int(server_port), int(backlog), target_address, int(target_port))
    core.gear.mainloop()









from socket import socket
from untwisted.event import READ, WRITE, EXPT, ERROR
from untwisted.dispatcher import *
from untwisted.core import die
from untwisted import core

class SuperSocket(Dispatcher):
    """
    The dispatching system for file descriptors.
    """

    def __init__(self, fd):
        Dispatcher.__init__(self)
        self.fd = fd
        core.gear.register(self)

    def add_map(self, event, handle, *args):
        Dispatcher.add_map(self, event, handle, *args)
        core.gear.scale(self)

    def del_map(self, event, handle, *args):
        Dispatcher.del_map(self, event, handle, *args)
        core.gear.scale(self)

    def destroy(self):
        self.base.clear()
        SSL.base.clear()
        del self.pool[:]
        core.gear.unregister(self)

class SSL(SuperSocket):
    """
    Dispatching system for SSL sockets.
    """
    def __init__(self, sock):
        self.sock = sock
        SuperSocket.__init__(self, sock.fileno())

    def __getattr__(self, name):
        return getattr(self.sock, name)

class Spin(socket, SuperSocket):
    """
    The dispatching system for sockets.
    """

    def __init__(self, sock=None):
        socket.__init__(self, _sock = sock._sock if sock else None)
        self.setblocking(0) 
        SuperSocket.__init__(self, self.fileno())

class Device(SuperSocket):
    """
    The dispatching system for child processes.
    """
    def __init__(self, device):
        from os import O_NONBLOCK
        from fcntl import fcntl, F_GETFL, F_SETFL
        
        self.device = device
        SuperSocket.__init__(self, self.device.fileno())

        # A non blocking device.
        fcntl(self.fd, F_SETFL, fcntl(self.fd, F_GETFL) | O_NONBLOCK)

    def __getattr__(self, name):
        return getattr(self.device, name)







""" 
"""

from untwisted.network import core, Spin, xmap
from untwisted.iostd import Client, lose, CONNECT, CONNECT_ERR
from untwisted.task import Task, DONE
from untwisted.network import die
from socket import *

def is_open(spin, port):
    print 'Port %s is open.' % port

def create_connection(addr, port):
    sock = socket(AF_INET, SOCK_STREAM)
    spin = Spin(sock)
    Client(spin)
    xmap(spin, CONNECT, is_open, port)
    spin.connect_ex((addr, port))
    return spin

def scan(addr, min, max):
    task = Task()
    for ind in xrange(min, max):
        task.add(create_connection(addr, ind), CONNECT, CONNECT_ERR)
    
    xmap(task, DONE, lambda task: die())

if __name__ == '__main__':
    from optparse import OptionParser
    parser   = OptionParser()
    parser.add_option("-a", "--addr", dest="addr", metavar="string", default='localhost')
    parser.add_option("-x", "--max", dest="max", metavar="integer", default=9999)
    parser.add_option("-n", "--min", dest="min", metavar="integer", default=70)

    (opt, args) = parser.parse_args()
    scan(opt.addr, int(opt.min), int(opt.max))
    core.gear.mainloop()
    











from subprocess import Popen, PIPE, STDOUT
from threading import Thread
from Queue import Queue, Empty
from os import environ 
from untwisted.dispatcher import Dispatcher
from untwisted import core
from untwisted.event import LOAD, CLOSE
from untwisted.waker import waker

class Expect(Thread, Dispatcher):
    """
    This class is used to spawn processes.

    python = Expect('python2.7', '-i')
    python.send('print "hello world"')
    python.terminate()
    python.destroy()
    """

    SIZE = 1024 * 124

    def __init__(self, *args):
        """
        """

        self.child     = Popen(args, shell=0, stdout=PIPE, stdin=PIPE,  
                                            stderr=STDOUT,  env=environ)
        self.stdin     = self.child.stdin
        self.stdout    = self.child.stdout
        self.stderr    = self.child.stderr
        self.args      = args
        self.queue     = Queue()
        self.terminate = self.child.terminate
        core.gear.pool.append(self)

        Dispatcher.__init__(self)
        Thread.__init__(self)

        self.start()

    def send(self, data):
        """
        Send data to the child process through.
        """

        self.stdin.write(data)

    def run(self):
        """
        """

        while self.feed():
            waker.wake_up()
        self.child.wait()

    def feed(self):
        try:
            data = self.stdout.readline()
        except Exception as e:
            data = ''
        finally:
            self.queue.put_nowait(data)
            return data

    def update(self):
        """
        """
        while not self.queue.empty():
            self.dispatch()

    def dispatch(self):
        data = self.queue.get_nowait()
        if not data: 
            self.drive(CLOSE)
        else: 
            self.drive(LOAD, data)

    def destroy(self):
        """
        Unregister up from untwisted reactor. It is needed
        to call self.terminate() first to kill the process.
        """
        core.gear.pool.remove(self)    
        self.base.clear()








from untwisted import core
import bisect
import time

class CancelCall(Exception):
    pass

class Timer(object):
    base = []
    def __init__(self, inc, callback, *args, **kwargs):
        self.args         = args
        self.kwargs       = kwargs
        self.inc          = inc
        self.time         = time.time()
        self.callback     = callback
        core.gear.pool.append(self)
        bisect.insort(Timer.base, inc)
        self.set_reactor_timeout()

    def update(self):
        if not time.time() - self.time > self.inc:
            return
        self.callback(*self.args, **self.kwargs)
        self.cancel()

    def cancel(self):
        core.gear.pool.remove(self)
        Timer.base.remove(self.inc)
        self.set_reactor_timeout()

    def set_reactor_timeout(self):
        core.gear.timeout = Timer.base[0] if Timer.base else core.gear.default_timeout


class Sched(Timer):
    def update(self):
        if not time.time() - self.time > self.inc:
            return

        try:
            self.callback(*self.args, **self.kwargs)
        except CancelCall:
            self.cancel()
        else:
            self.time = time.time()






""" deb_4.py 
    Usage: 
"""

from untwisted.network import *
from untwisted.utils.stdio import *
from untwisted.utils.shrug import *
from untwisted.task import Task, COMPLETE
import sys
import random
MAX_REQUEST = 10000
PORT = 1234

def on_accept(serv, con):
    Stdin(con)
    Stdout(con)
    
    base = range(0, MAX_REQUEST)
    random.shuffle(base)

    for ind in base:
        con.dump('%s\r\n' % ind)
    
    xmap(con, DUMPED, lose)

def set_up_server():
    sock = socket(AF_INET, SOCK_STREAM)
    serv = Spin(sock)
    
    serv.bind(('', PORT))
    serv.listen(5)
    Server(serv)

    xmap(serv, ACCEPT, on_accept)

def done(task, data):
    print data
    raise Kill

def on_connect(con):
    Stdin(con)
    Stdout(con)
    Shrug(con)
    xmap(con, FOUND, lambda con, data: spawn(con, data, data))
    #xmap(con, FOUND, lambda con, data: sys.stdout.write('%s\n' % data))
    print 'connected' 

    task = Task(dict())
    
    def job(data, event, args):
        base = data.setdefault(event, list())
        base.append(args[1])
        return True

    for ind in xrange(MAX_REQUEST):
        task.gather(con, ('%s' % ind, job)) 

    xmap(task, COMPLETE, done)

def main():
    set_up_server()

    sock = socket(AF_INET, SOCK_STREAM)
    con = Spin(sock)
    Client(con)

    xmap(con, CONNECT, on_connect)
    xmap(con, CONNECT_ERR, on_connect_err)
    con.connect_ex(('localhost', PORT))


def on_connect_err(con, err):
    print 'on_connect_err %s.' % err

if __name__ == '__main__':
    main()
    #install_reactor(core.Select)
    core.gear.mainloop()




"""

"""

# We import the basic modules.
from untwisted.network import *
from untwisted.iostd import *
from untwisted.splits import Terminator, logcon
from untwisted.tools import coroutine
from untwisted.plugins.irc import *
from socket import *
from os.path import getsize, isfile

ADDRESS = 'irc.freenode.org'
PORT = 6667
NICK = 'uxirc'
USER = 'uxirc uxirc uxirc :uxirc'

# The folder where we will be serving files.
FOLDER = '/home/tau/Downloads'

@coroutine
def get_myaddr(con, prefix, nick, msg):
    send_cmd(con, 'userhost %s' % nick)
    args        = yield con, '302'
    _, _, ident = args
    user, myaddr   = ident.split('@')
    con.myaddr  = myaddr

def setup(con):
    Stdin(con)
    Stdout(con)
    Terminator(con)
    Irc(con)
    CTCP(con)

    xmap(con, 'PING', lambda con, prefix, servaddr: send_cmd(con, 'PONG :%s' % servaddr))
    xmap(con, '376', lambda con, *args: send_cmd(con, 'JOIN #ameliabot'))

    xmap(con, 'PRIVMSG', send_file)
    xmap(con, 'DCC SEND', check_file_existence)
    xmap(con, '376', get_myaddr)
    xmap(con, CLOSE, lambda con, err: lose(con))

    logcon(con)

    send_cmd(con, 'NICK %s' % NICK)
    send_cmd(con, 'USER %s' % USER)

def main():
    sock = socket(AF_INET, SOCK_STREAM)
    con  = Spin(sock)

    Client(con)
    con.connect_ex((ADDRESS, PORT))
    xmap(con, CONNECT, setup) 

def send_file(con, nick, user, host, target, msg):
    if not msg.startswith('.send'): 
        return
        
    cmd, filename, port = msg.split(' ')
    resource            = '%s/%s' % (FOLDER, filename)
    size                = getsize(resource)
    fd                  = open(resource, 'rb')

    def is_done(msg):
        send_msg(con, nick, msg)
        fd.close()

    try:
        dcc = DccServer(fd, int(port), timeout=50)
    except Exception:
        is_done('Couldnt list on the port!')
        raise

    xmap(dcc, DONE, lambda *args: is_done('Done.'))
    xmap(dcc, CLOSE, lambda *args: is_done('Failed!'))
    xmap(dcc, ACCEPT_ERR, lambda *args: is_done("Accept error!"))
    xmap(dcc, TIMEOUT, lambda *args: is_done('TIMEOUT!'))    

    HEADER  = '\001DCC SEND %s %s %s %s\001' 
    request = HEADER % (filename, ip_to_long(con.myaddr), port, size)
    send_msg(con, nick, request)

def check_file_existence(con, (nick, user, host, target, msg), 
                                        filename, address, port, size):
    resource = '%s/%s' % (FOLDER, filename) 
    if isfile(resource):      
        send_msg(con, nick, 'File already exists.')
    else:
        recv_file(con, nick, resource, address, port, size)

def recv_file(con, nick, resource, address, port, size):
    fd  = open(resource, 'wb')
    dcc = DccClient(long_to_ip(int(address)), 
                     int(port), fd, int(size)) 

    def is_done(msg):
        send_msg(con, nick, msg)
        fd.close()

    xmap(dcc, DONE, lambda *args: is_done('Done!'))
    xmap(dcc, CLOSE, lambda *args: is_done('Failed!'))
    xmap(dcc, CONNECT_ERR, lambda *args: is_done("It couldn't connect!"))

if __name__ == '__main__':
    # import argparser
    # parser = argparser.ArgumentParser()
    main()
    core.gear.mainloop()





"""
    Name: deb_2.py
    Description: It is a unit test for DccClient, DccServer classes.
    It checkes whether the TIMEOUT event is being spawned correctly.
"""

from untwisted.network import *
from untwisted.event import *
from uxirc.dcc import *
from untwisted.task import sched
from StringIO import StringIO
import sys
logvar = sys.stdout

def run_test(n, m):
    # There will be no transmission
    # of data.
    size = 0
    for ind in xrange(n, m):
        # We will not be ending data
        # so it is empty.
        fd = StringIO() 
        DccServer(fd, ind)
   
    def connect():
        for ind in xrange(n, m):
            fd = StringIO()
            # It will attempt to connect.
            client = DccClient('localhost', ind, fd, size)
            
            # If it fails then the DccServer spawned
            # TIMEOUT correctly and there is no listening socket.
            xmap(client,  
                 CONNECT_ERR, 
                 lambda x, ident=ind: logvar.write('Connection fail %s\n' % ident))
            # If it connects then we have done something wrong.
            xmap(client,  
                 CONNECT, 
                 lambda x, ident=ind: logvar.write('Connected %s\n' % ident))

    # It calls connect after 25 the timeout should run in 20.
    sched.after(25, connect, True)

def main():
    # If it happens connection fail
    # over all the clients then it is
    # fine.
    run_test(1234, 1400)
    gear.mainloop()

main()



""" 
"""

from untwisted.splits import Terminator, Fixed
from untwisted.iputils import ip_to_long, long_to_ip
from untwisted.network import *
from untwisted.dispatcher import Dispatcher
from untwisted.iostd import *
from untwisted.timer import Timer
from untwisted.event import TIMEOUT, DONE
from struct import pack, unpack
from textwrap import wrap
from socket import *

import re
import sys

RFC_STR        = "^(:(?P<prefix>[^ ]+) +)?(?P<command>[^ ]+)( *(?P<arguments> .+))?"
RFC_REG        = re.compile(RFC_STR)
PREFIX_STR     = "(?P<nick>.+)!(?P<user>.+)@(?P<host>.+)"
PREFIX_REG     = re.compile(PREFIX_STR)
PRIVMSG_HEADER = 'PRIVMSG %s :%s\r\n'
CMD_HEADER = '%s\r\n'

class DccServer(Dispatcher):
    """ 
    This class is used to send files. It is called DccServer cause the one sending 
    a file is the one who sets up a server.
    """

    def __init__(self, fd, port, timeout=20):
        """
        fd      -> The file to be sent.
        port    -> The port which will be used.
        timeout -> How long the server must be up.
        """

        sock = socket(AF_INET, SOCK_STREAM)
        sock.bind(('', port))
        sock.listen(1)
        self.local = Spin(sock)
        Server(self.local)

        Dispatcher.__init__(self)
        self.fd      = fd
        self.timeout = timeout
        self.port    = port
       
        xmap(self.local, ACCEPT, self.on_accept) 
        self.timer = Timer(self.timeout, self.on_timeout)
        
    def on_timeout(self):
        spawn(self, TIMEOUT)
        lose(self.local)

    def on_accept(self, local, spin):
        """
        """

        Stdout(spin)
        Stdin(spin)
        Fixed(spin)

        spin.dumpfile(self.fd)

        xmap(spin, CLOSE, lambda con, err: lose(con)) 
        xmap(spin, Fixed.FOUND, self.on_ack) 
        spin.add_handle(lambda spin, event, args: spawn(self, event, spin, *args))
        self.timer.cancel()

    def on_ack(self, spin, ack):
        """
        """
        pos = unpack("!I", ack)[0]

        if pos >= self.fd.tell(): 
            self.run_done(spin)

    def run_done(self, spin):
        lose(spin)
        lose(self.local)
        spawn(spin, DONE)

class DccClient(Dispatcher):
    def __init__(self, host, port, fd, size):
        """
        """
        Dispatcher.__init__(self)

        sock      = socket(AF_INET, SOCK_STREAM)
        spin      = Spin(sock)
        self.port = port
        self.fd   = fd
        self.size = size
        
        Client(spin)
        spin.connect_ex((host, port))
   
        xmap(spin, CONNECT, self.on_connect) 
        xmap(spin, CONNECT_ERR, lambda con, err: lose(con)) 

        spin.add_handle(lambda spin, event, args: spawn(self, event, spin, *args))

    def on_connect(self, spin):
        """
        """

        Stdout(spin)
        Stdin(spin)
        xmap(spin, LOAD, self.on_load) 
        xmap(spin, CLOSE, lambda con, err: lose(con)) 

    def on_load(self, spin, data):
        """
        """

        self.fd.write(data)
        spin.dump(pack('!I', self.fd.tell()))

        if self.fd.tell() >= self.size:
            xmap(spin, DUMPED, self.run_done)

    def run_done(self, spin):
        lose(spin)
        spawn(self, DONE)

class Irc(object):
    def __init__(self, spin):
        """ 
        Install the protocol inside a Spin instance. 
        """
        xmap(spin, Terminator.FOUND, self.main)

    def main(self, spin, data):
        """ 
        The function which uses irc rfc regex to extract
        the basic arguments from the msg.
        """

        field = re.match(RFC_REG, data)
        
        if not field:
            return
    
        prefix  = self.extract_prefix(field.group('prefix'))
        command = field.group('command').upper()
        args    = self.extract_args(field.group('arguments'))
        spawn(spin, command, *(prefix + args))
    
    def extract_prefix(self, prefix):
        """ 
        It extracts an irc msg prefix chunk 
        """

        field = re.match(PREFIX_REG, prefix if prefix else '')
        
        return (prefix,) if not field else field.group(1, 2, 3)
    
    def extract_args(self, data):
        """ 
        It extracts irc msg arguments. 
        """
        args = []
        data = data.strip(' ')
        if ':' in data:
            lhs, rhs = data.split(':', 1)
            if lhs: args.extend(lhs.rstrip(' ').split(' '))
            args.append(rhs)
        else:
           args.extend(data.split(' '))
        return tuple(args)    
    
class CTCP(object):
    def __init__(self, spin):
        """ 
        It installs the subprotocol inside a Spin
        instance.
        """
        xmap(spin, 'PRIVMSG', self.extract_ctcp)
    
        xmap(spin, 'DCC', self.patch)
    
    def extract_ctcp(self, spin, nick, user, host, target, msg):
        """ 
        it is used to extract ctcp requests into pieces.
        """
    
        # The ctcp delimiter token.
        DELIM = '\001'
    
        if not msg.startswith(DELIM) or not msg.endswith(DELIM):
            return
    
        ctcp_args = msg.strip(DELIM).split(' ') 
        
        spawn(spin, ctcp_args[0], (nick, user, host,  target, msg), *ctcp_args[1:])
    
    def patch(self, spin, header, *args):
        """ 
        It spawns DCC TYPE as event. 
        """

        spawn(spin, 'DCC %s' % args[0], header, *args[1:])
    

class Misc(object):
    def __init__(self, spin):
        xmap(spin, '001', self.on_001)
        xmap(spin, 'PRIVMSG', self.on_privmsg)
        xmap(spin, 'JOIN', self.on_join)
        xmap(spin, 'PART', self.on_part)
        xmap(spin, '353', self.on_353)
        xmap(spin, '332', self.on_332)
        xmap(spin, 'NICK', self.on_nick)
        xmap(spin, 'KICK', self.on_kick)
        xmap(spin, 'MODE', self.on_mode)

        self.nick = ''

    def on_privmsg(self, spin, nick, user, host, target, msg):
        spawn(spin, 'PRIVMSG->%s' % target.lower(), nick, user, host, msg)
        spawn(spin, 'PRIVMSG->%s' % nick.lower(), target, user, host, msg)

        if target.startswith('#'):
            spawn(spin, 'CMSG', nick, user, host, target, msg)
        elif self.nick.lower() == target.lower():
            spawn(spin, 'PMSG', nick, user, host, target, msg)
        
    def on_join(self, spin, nick, user, host, chan):
        if self.nick == nick: 
            spawn(spin, '*JOIN', chan)
        else:
            spawn(spin, 'JOIN->%s' % chan, nick, 
                  user, host)
    
    def on_353(self, spin, prefix, nick, mode, chan, peers):
        spawn(spin, '353->%s' % chan, prefix, 
              nick, mode, peers)
    
    def on_332(self, spin, addr, nick, channel, msg):
        spawn(spin, '332->%s' % channel, addr, nick, msg)
    
    def on_part(self, spin, nick, user, host, chan, msg=''):
        spawn(spin, 'PART->%s' % chan, nick, 
              user, host, msg)
    
        if self.nick == nick: 
            spawn(spin, '*PART->%s' % chan, user, host, msg)
    
    def on_001(self, spin, address, nick, *args):
        self.nick = nick
    
    def on_nick(self, spin, nicka, user, host, nickb):
        if not self.nick == nicka: 
            return
    
        self.nick = nickb;
        spawn(spin, '*NICK', nicka, user, host, nickb)

    def on_kick(self, spin, nick, user, host, chan, target, msg=''):
        spawn(spin, 'KICK->%s' % chan, nick, user, host, target, msg)

        if self.nick == target:
            spawn(spin, '*KICK->%s' % chan, nick, user, host, target, msg)

    def on_mode(self, spin, nick, user, host, chan='', mode='', target=''):
        spawn(spin, 'MODE->%s' % chan, nick, user, host, mode, target)

def send_msg(server, target, msg):
    for ind in wrap(msg, width=512):
        server.dump(PRIVMSG_HEADER % (target, ind))

def send_cmd(server, cmd):
    server.dump(CMD_HEADER % cmd)




















from untwisted.dispatcher import Dispatcher
from untwisted.event import DONE

class Task(Dispatcher):
    """
    """

    def __init__(self):
        Dispatcher.__init__(self)
        self.count = 0

    def add(self, dispatcher, *events):
        """
        """

        self.count = self.count + 1
        for ind in events:
            dispatcher.add_map(ind, self.is_done, events)

    def is_done(self, dispatcher, *args):
        self.count = self.count - 1

        for ind in args[-1]:
            dispatcher.del_map(ind, self.is_done, args[-1])

        if not self.count: 
            self.drive(DONE)







# It imports basic objects.
from untwisted.network import core, Spin, xmap
from untwisted.iostd import Server, Stdin, ACCEPT, CLOSE
from time import asctime

class TimeServ(object):
    def __init__(self, server):
        xmap(server, ACCEPT, self.handle_accept)
       

    def handle_accept(self, server, con):
        Stdin(con)

        con.dump('%s\r\n' % asctime())
        xmap(con, CLOSE, lambda con, err: lose(con))


if __name__ == '__main__':
    server = Spin()
    server.bind(('', 1234))
    server.listen(200)

    Server(server)
    TimeServ(server)
    core.gear.mainloop()








"""
"""

from untwisted.network import core, Spin, xmap, spawn
from untwisted.iostd import Server, Stdin, Stdout, CLOSE, ACCEPT
from untwisted.splits import Terminator
from untwisted.tools import handle_exception
import operator

class CalcParser(object):
    """
    """

    def __init__(self, client):
        xmap(client, Terminator.FOUND, self.handle_found)

    @handle_exception(ValueError)
    def handle_found(client, data):
        op, args = data.split(' ', 1)
        args     = map(float, args.split(' '))
        spawn(client, op, args)

class CalcServer(object):
    """ 
    """

    def __init__(self, server):
        xmap(server, ACCEPT, self.handle_accept)

    def handle_accept(self, server, client):
        Stdin(client)
        Stdout(client)
        Terminator(client, delim='\r\n')
        parser = CalcParser(client)
        
        xmap(client, 'add', self. on_add)    
        xmap(client, 'sub', self.on_sub)    
        xmap(client, 'mul', self.on_mul)    
        xmap(client, 'div', self.on_div)    
        xmap(client, (parser.handle_found, ValueError), self.on_error)    

        xmap(client, CLOSE, self.handle_close)

    def on_add(self, client, args):
        self.send_msg(client, reduce(operator.add, args, 0))

    def on_sub(self, client, args):
        self.send_msg(client, reduce(operator.sub, args, 0))

    def on_div(self, client, args):
        self.send_msg(client, reduce(operator.div, args, args.pop(0)))

    def on_mul(self, client, args):
        self.send_msg(client, reduce(operator.mul, args, args.pop(0)))

    def on_error(self, client, excpt):
        self.send_msg(client, excpt)

    def handle_close(self, client, err):
        client.destroy()
        client.close()

    def send_msg(self, client, msg):
        client.dump('%s\r\n' % msg)

if __name__ == '__main__':
    server = Spin()
    server.bind(('', 1234))
    server.listen(5)

    Server(server)
    CalcServer(server)
    core.gear.mainloop()








from untwisted.network import xmap, Spin, core
from untwisted.iostd import CLOSE, LOAD, CONNECT, CONNECT_ERR, Stdout, Stdin, Client, lose
from untwisted.splits import Terminator, logcon
from untwisted.plugins.irc import Irc, send_cmd, send_msg
from socket import *

class FunBot(object):
    """ Bot class """
    def __init__(self, ip, port, nick, user, password, *chan_list):
        """ It sets up the bot instance. """
        sock = socket(AF_INET, SOCK_STREAM)

        # We have to wrap our socket with a Spin instance
        # in order to have our events issued when data comes
        # from the socket.
        con = Spin(sock)
        
        # This protocol is required by uxirc.irc protocol.
        # It spawns CONNECT.
        Client(con)

        # We use connect_ex since we do not want an exception.
        # Untwisted uses non blocking sockets.
        con.connect_ex((ip, port))

        # We save whatever we might need.
        self.nick      = nick
        self.user      = user
        self.password  = password
        self.chan_list = chan_list
        self.ip        = ip
        self.port      = port

        # It maps CONNECT to self.send_auth so
        # when our socket connects it sends NICK and USER info.
        xmap(con, CONNECT, self.send_auth)
        xmap(con, CONNECT_ERR, lambda con, err: lose(con))

    def send_auth(self, con):
        # It is what we use to send data. send_msg function uses
        # spin.dump function to dump commands.
        Stdin(con)

        # Shrug protocols requires Stdout that spawns LOAD
        # when data arrives. 
        Stdout(con)

        # This protocol spawns FOUND whenever it finds \r\n.
        Terminator(con)

        Irc(con)
        
        xmap(con, CLOSE, lambda con, err: lose(con))

        # Now, it basically means: when it '376' irc command is
        # issued by the server then calls self.auto_join.
        # We use auto_join to send the sequence of JOIN
        # commands in order to join channels.
        xmap(con, '376', self.auto_join)

        # Below the remaining stuff follows the same pattern.
        xmap(con, 'JOIN', self.on_join)
        xmap(con, 'PING', self.on_ping)
        xmap(con, 'PART', self.on_part)
        xmap(con, '376', self.on_376)
        xmap(con, 'NOTICE', self.on_notice)
        xmap(con, 'PRIVMSG', self.on_privmsg)
        xmap(con, '332', self.on_332)
        xmap(con, '001', self.on_001)
        xmap(con, '001', self.on_002)
        xmap(con, '003', self.on_003)
        xmap(con, '004', self.on_004)
        xmap(con, '333', self.on_333)
        xmap(con, '353', self.on_353)
        xmap(con, '366', self.on_366)
        xmap(con, '474', self.on_474)
        xmap(con, '302', self.on_302)


        send_cmd(con, 'NICK %s' % self.nick)
        send_cmd(con, 'USER %s' % self.user)
        send_msg(con, 'nickserv', 'identify %s' % self.password)

    def auto_join(self, con, *args):
        for ind in self.chan_list:
            send_cmd(con, 'JOIN %s' % ind)

    def on_ping(self, con, prefix, servaddr):
        # If we do not need pong we are disconnected.
        print 'on_ping', (prefix, servaddr)
        reply = 'PONG :%s\r\n' % servaddr
        send_cmd(con, reply)
        
    def on_join(self, con, nick, user, host, chan):
        print 'on_join\n', (nick, user, host, chan)

    def on_part(self, con, nick, user, host, chan):
        print 'on_part\n', (nick, user, host, chan)

    def on_privmsg(self, con, nick, user, host, target, msg):
        print 'on_privmsg\n', (nick, user, host, target, msg)

    def on_332(self, con, prefix, nick, chan, topic):
        print 'on_332\n', (prefix, nick, chan, topic)

    def on_302(self, con, prefix, nick, info):
        print 'on_302\n', (prefix, nick, info)

    def on_333(self, con, prefix, nick_a, chan,  nick_b, ident):
        print 'on_333\n', (prefix, nick_a, chan, nick_b, ident)

    def on_353(self, con, prefix, nick, mode, chan, peers):
        print 'on_353\n', (prefix, nick, mode, chan, peers)

    def on_366(self, con, prefix, nick, chan, msg):
        print 'on_366\n', (prefix, nick, chan, msg)

    def on_474(self, con, prefix, nick, chan, msg):
        print 'on_474\n', (prefix, nick, chan, msg)

    def on_376(self, con, prefix, nick, msg):
        print 'on_376\n', (prefix, nick, msg)

    def on_notice(self, con, prefix, nick, msg, *args):
        print 'on_notice\n', (prefix, nick, msg), args

    def on_001(self, con, prefix, nick, msg):
        print 'on_001\n', (prefix, nick, msg)

    def on_002(self, con, prefix, nick, msg):
        print 'on_002\n', (prefix, nick, msg)

    def on_003(self, con, prefix, nick, msg):
        print 'on_004\n', (prefix, nick, msg)

    def on_004(self, con, prefix, nick, *args):
        print 'on_004\n', (prefix, nick, args)
    
    def on_005(self, con, prefix, nick, *args):
        print 'on_005', (prefix, nick, args)

bot = FunBot('irc.freenode.com', 6667, 'Fourier1', 'kaus keus kius :kous', '', '##calculus')
core.gear.mainloop()








from untwisted.dispatcher import Erase

class Hold(object):
    def __init__(self, seq):
        self.seq                    = seq
        self.dispatcher, self.event = next(seq)
        self.dispatcher.add_map(self.event, self)

    def __call__(self, dispatcher, *args):
        try:
            dispacher, event = self.seq.send(args)
        except StopIteration:
            self.dispatcher.del_map(self.event, self)
        else:
            if self.dispatcher != dispatcher or event != self.event:
                self.swap_dispatcher(event, dispatcher)

    def swap_dispatcher(self, event, dispatcher):
        self.dispatcher.del_map(self.event, self.dispatcher)
        dispatcher.add_map(event, self)
        self.dispatcher  = dispatcher
        self.event       = event

class HandleProcessor(object):
    def __call__(self, handle):
        self.handle = handle
        return self.process_handle

    def __init__(self, *excpts):
        self.excpts = excpts

class handle_exception(HandleProcessor):
    def process_handle(self, dispatcher, *args):
        try:
            value = self.handle(dispatcher, *args)
        except self.excpts as e:
            dispatcher.drive((self.process_handle, e.__class__), e)

class mapcall(HandleProcessor):
    def process_handle(self, dispatcher, *args):
        try:
            value = self.handle(dispatcher, *args)
        except self.excpts as e:
            dispatcher.drive((self.process_handle, e.__class__), e)
        else:
            dispatcher.drive(self.process_handle, value)

def coroutine(handle):
    def start(*args, **kwargs):
        hold = Hold(handle(*args, **kwargs))
    return start





from untwisted.iostd import LOAD, CLOSE, CONNECT, CONNECT_ERR, Client, Stdin, Stdout, lose
from untwisted.splits import AccUntil, TmpFile
from untwisted.network import Spin, xmap, spawn, zmap
from untwisted.event import get_event
from urllib import urlencode
from untwisted.plugins import rapidserv
from tempfile import TemporaryFile as tmpfile

DEFAULT_HEADERS = {
    'user-agent':"Untwisted-requests/1.0.0", 
    'accept-charset':'ISO-8859-1,utf-8;q=0.7,*;q=0.7',
    'connection':'close',
    }

class Response(object):
    def __init__(self, data):
        headers                              = data.split('\r\n')
        response                             = headers.pop(0)
        self.version, self.code, self.reason = response.split(' ', 2)
        self.headers                         = rapidserv.Headers(headers)
        self.fd                              = tmpfile('a+')

class HttpTransferHandle(rapidserv.HttpTransferHandle):
    def process_request(self, spin, response, data):
        """
        """

        response = Response(response)
        spawn(spin, HttpTransferHandle.HTTP_TRANSFER, response, data)

class HttpResponseHandle(rapidserv.HttpRequestHandle):
    HTTP_RESPONSE = rapidserv.HttpRequestHandle.HTTP_REQUEST

class HttpCode(object):
    def __init__(self, spin):
        xmap(spin, HttpResponseHandle.HTTP_RESPONSE, self.process)

    def process(self, spin, response):
        pass

def on_connect(spin, request):
    Stdin(spin)
    Stdout(spin)
    AccUntil(spin)
    HttpTransferHandle(spin)

    # It has to be mapped here otherwise HttpTransferHandle.HTTP_RESPONSE
    # will be spawned and response.fd cursor will be at the end of the file.
    xmap(spin, TmpFile.DONE, 
                        lambda spin, fd, data: fd.seek(0))

    HttpResponseHandle(spin)
    xmap(spin, CLOSE, lambda con, err: lose(con))

    spin.dump(request)

def create_connection(addr, port, request):
    con  = Spin()
    Client(con)
    con.connect_ex((addr, port))

    xmap(con, CONNECT,  on_connect, request)
    xmap(con, CONNECT_ERR, lambda con, err: lose(con))
    return con

def get(addr, port, path, args={}, version='HTTP/1.1', headers=DEFAULT_HEADERS):
    args = '?%s' % urlencode(args) if args else ''
    headers['host'] = addr

    data  = 'GET %s%s %s\r\n' % (path, args, version)

    for key, value in headers.iteritems():
        data = data + '%s: %s\r\n' % (key, value)
    data = data + '\r\n'
    spin = create_connection(addr, port, data)
    return spin

def post(addr, port, path, payload={}, version='HTTP/1.1', headers=DEFAULT_HEADERS):
    payload                  = urlencode(payload)
    request                  = 'POST %s %s\r\n' % (path, version)
    # should be fixed the content type thing.
    headers['host'] = addr

    headers['content-type']   = 'application/x-www-form-urlencoded'
    headers['content-length'] = len(payload)

    for key, value in headers.iteritems():
        request = request + '%s: %s\r\n' % (key, value)
    request = request + '\r\n' + payload
    spin    = create_connection(addr, port, request)
    return spin

def auth(username, password):
    from base64 import encodestring
    base = encodestring('%s:%s' % (username, password))
    base = base.replace('\n', '')
    return "Basic %s" % base





""" deb_1.py 
    Usage: Used to test hold object functionality.
    It uses the /demo/echo/echo.py to send commands and wait for them
    using hold object.
"""
from untwisted.network import *
from untwisted.utils.stdio import *
from untwisted.utils.shrug import *

def main():
    for ind in xrange(100):
        sock = socket(AF_INET, SOCK_STREAM)
        spin = Spin(sock)
        Client(spin)
        Stdin(spin)
        Stdout(spin)
        Shrug(spin)
        spin.connect_ex(('', 1234))
        xmap(spin, FOUND, split)
        xmap(spin, CONNECT, on_connect, ind)
        xmap(spin, CONNECT_ERR, on_connect_err, ind)


def split(spin, data):
    spawn(spin, data)

def on_connect(spin, counter):
    spin.dump('cmd_x\r\n')
    print 'Waiting for cmd_x from, ', counter
    event, args = yield hold(spin, 'cmd_x')
    print 'cmd_x arrived from, ', counter
    spin.dump('cmd_y\r\n')
    print 'Waiting for cmd_y from, ', counter
    event, args = yield hold(spin, 'cmd_y')
    print 'cmd_y arrived from, ', counter

def on_connect_err(spin, ind):
    print 'Error connect from, ', ind
    pass

if __name__ == '__main__':
    main()
    core.gear.mainloop()



from untwisted.network import xmap, Spin, core
from untwisted.iostd import Server, Stdout, Stdin, lose, ACCEPT, LOAD, CLOSE
from socket import socket, AF_INET, SOCK_STREAM
import sys

def setup(server, con):
    Stdin(con)
    Stdout(con)
    xmap(con, CLOSE, lambda con, err: lose(con))
    xmap(con, LOAD, lambda con, data: sys.stdout.write('%s\r\n' % data))

if __name__ == '__main__':    
    from optparse import OptionParser

    parser   = OptionParser()
    parser.add_option("-a", "--addr", dest="addr",
                      metavar="string", default='0.0.0.0')
                  
    parser.add_option("-p", "--port", dest="port",
                      type="int", default=1234)

    parser.add_option("-b", "--backlog", dest="backlog",
                      type="int", default=5)

    (opt, args) = parser.parse_args()
    sock   = socket(AF_INET, SOCK_STREAM)
    server = Spin(sock) 
    server.bind((opt.addr, opt.port))
    server.listen(opt.backlog)
    Server(server)
    
    xmap(server, ACCEPT, setup)
    
    core.gear.mainloop()











""" deb_1.py 
    Usage: Used to test untwisted thread support.
    It starts a connection with a server then the server
    launches threads that waits 3 seconds
    and return a random number. the register
    function returns an ident for the thread
    thats used with cmap to map the ident thread
    to a callback when the thread finishes its task
    and returns a value it calls the functions
    mapped to the thread ident thats a string
    returned by thread.getName().
    the job instance is pretty much a Spin instance
    where events are thread name processes.
"""

from untwisted.network import *
from untwisted.utils.stdio import *
from untwisted.utils.shrug import *
from untwisted.job import job
import time

import sys

PORT = 1235

count = 0

def demo(index):
    time.sleep(3)
    return index


def check(job, data, con):
    global count
    con.dump('%s count: %s\r\n' % (data, count))
    count = count + 1

def on_accept(serv, con):
    Stdin(con)
    Stdout(con)
    
    for ind in xrange(1000):
        ident = job.register(demo, ind)
        # Obs i could use xmap but i would have to use zmap again
        # otherwise it would contain a handle that wouldnt be called
        # again. cmap just removes the handle when it is executed once.
        cmap(job, ident, check, con)


def set_up_server():
    sock = socket(AF_INET, SOCK_STREAM)
    serv = Spin(sock)
    
    serv.bind(('', PORT))
    serv.listen(300)
    Server(serv)

    xmap(serv, ACCEPT, on_accept)

def on_connect(con):
    Stdin(con)
    Stdout(con)
    Shrug(con)
    xmap(con, FOUND, lambda con, data: sys.stdout.write('%s\n' % data))
    print 'connected' 

def main():
    set_up_server()

    sock = socket(AF_INET, SOCK_STREAM)
    con = Spin(sock)
    Client(con)

    xmap(con, CONNECT, on_connect)
    xmap(con, CONNECT_ERR, on_connect_err)
    con.connect_ex(('localhost', PORT))


def on_connect_err(con, err):
    print 'on_connect_err %s.' % err

if __name__ == '__main__':
    main()
    #install_reactor(core.Select)
    core.gear.mainloop()




#! /usr/bin/env python2

from distutils.core import setup
setup(name="untwisted",
      version="1.0.0",
      packages=["untwisted", 
                "untwisted.plugins"],
      scripts=['quickserv'],
      author="Iury O. G. Figueiredo",
      author_email="ioliveira@id.uff.br",
      url='https://github.com/iogf/untwisted',
      download_url='https://github.com/iogf/untwisted/releases',
      keywords=['asynchronous', 'programming', 'twisted', 'untwisted', 'library', 'framework', 'networking', 'protocols', 'rapidserv', 'irc', 'requests'],
      classifiers=[],
      description="A library for asynchronous programming in python.",)



















from untwisted.network import *
from untwisted.iostd import Client, Stdin, Stdout, CLOSE, CONNECT, CONNECT_ERR, lose
from untwisted.splits import Terminator, logcon
from untwisted.plugins.irc import Irc, send_cmd
from socket import *

def send_auth(con, nick, user):
    Stdin(con)
    Stdout(con)
    Terminator(con)
    Irc(con)
    logcon(con)

    xmap(con, CLOSE, lambda con, err: lose(con))
    send_cmd(con, 'NICK %s' % nick)
    send_cmd(con, 'USER %s' % user)

def send_pong(con, prefix, servaddr):
    reply = 'PONG :%s\r\n' % servaddr
    send_cmd(con, reply)

def connect(addr, port, nick, user, *chans):
    sock = socket(AF_INET, SOCK_STREAM)
    con  = Spin(sock)
    Client(con)
    con.connect_ex((addr, port))

    def auto_join(con, *args):
        for ind in chans:
            send_cmd(con, 'JOIN %s' % ind)

    xmap(con, CONNECT, send_auth, nick, user)
    xmap(con, CONNECT_ERR, lambda con, err: lose(con))
    xmap(con, 'PING', send_pong)
    xmap(con, '376', auto_join)
    return con

NICK_LIST = ('alphaaaa','cooool', 'blablalb')

for ind in NICK_LIST:
    connect('irc.freenode.org', 6667, ind, 'ae eu de :uxirc', '#vy')

core.gear.mainloop()






from untwisted.network import core, Spin, xmap, die
from untwisted.iostd import Client, CONNECT, CONNECT_ERR
import errno

def on_connect(con, addr, port):
    print 'Connected to %s:%s !' % (addr, port)

def on_connect_err(con, err, addr, port):
    print "Failed to connect to %s:%s, errcode " % (addr, port), errno.errorcode[err]

def create_connection(addr, port):
    con = Spin()
    Client(con)
    xmap(con, CONNECT, on_connect, addr, port)
    xmap(con, CONNECT_ERR, on_connect_err, addr, port)
    xmap(con, CONNECT, lambda con: die())
    xmap(con, CONNECT_ERR, lambda con, err: die())
    
    con.connect_ex((addr, port))

if __name__ == '__main__':
    import argparse
    parser = argparse.ArgumentParser()
    parser.add_argument('-a', '--addr', help='Address')
    parser.add_argument('-p', '--port', type=int, help='Port')
    args = parser.parse_args()

    create_connection(args.addr, args.port)
    core.gear.mainloop()
    






from untwisted.expect import Expect, LOAD, CLOSE
from untwisted.network import core, xmap, die

def handle(expect, data):
    print data

expect = Expect('python', '-i', '-u')
xmap(expect, LOAD, handle)
xmap(expect, CLOSE, lambda expect: die())
expect.send('print "hello world"\nquit()\n')
core.gear.mainloop()







"""
"""

from untwisted.plugins.requests import HttpResponseHandle, post
from untwisted.network import Spin, xmap, core, die
import argparse

def on_done(spin, response):
    print 'URL:%s' % response.headers['location']
    die()

if __name__ == '__main__':
    parser= argparse.ArgumentParser()
    parser.add_argument('-f', '--filename',  default='0.0.0.0', help='filename')
    parser.add_argument('-t', '--type',  default='Plain Text', 
                                help='Type should be Python, Haskell, etc..')
    parser.add_argument('-r', '--run',  action='store_true', help='run')
    args = parser.parse_args()

    fd   = open(args.filename, 'r')
    code = fd.read()
    fd.close()

    payload = {
                    'code':code,
                    'lang':' '.join(map(lambda ind: ind.capitalize(), 
                                        args.type.split(' '))),
                    'submit':'Submit',
                    'run': args.run
              }
    
    con = post('codepad.org', 80, '/', payload)
    xmap(con, HttpResponseHandle.HTTP_RESPONSE, on_done)
    core.gear.mainloop()









"""

"""

from untwisted.plugins.rapidserv import RapidServ, make, HttpRequestHandle
import shelve

DB_FILENAME = 'DB'
DB          = shelve.open(make(__file__, DB_FILENAME))
HttpRequestHandle.MAX_SIZE = 1024 * 1024 * 3
app    = RapidServ(__file__)

@app.overflow
def response(con, request):
    con.set_response('HTTP/1.1 400 Bad request')
    HTML = '<html> <body> <h1> Bad request </h1> </body> </html>'
    con.add_data(HTML)
    con.done()

@app.route('GET /')
def index(con):
    con.render('view.jinja', posts = DB.iterkeys())
    con.done()

@app.route('GET /load_index')
def load_index(con, index):
    con.add_data(DB[index[0]], mimetype='image/jpeg')
    con.done()

@app.route('POST /add_image')
def add_image(con, file):
    DB[file.filename] = file.file.read()
    index(con)

if __name__ == '__main__':
    app.run()






from threading import Thread, Lock
from untwisted import core
from untwisted.waker import waker
from untwisted.event import DONE, ERROR
from untwisted.dispatcher import Dispatcher

class Job(Thread, Dispatcher):
    """
    This class is used to execute a function inside a thread.

        def func(a, b):
            return a + b
    
        def on_done(job, sum):
            print sum
    
        job = Job(func, 1, 2)
        xmap(job, DONE, on_done)
    
    In case of an exception occuring inside func then the event 
    ERROR happens instead of the DONE event which carries the exception thrown.    
    """

    def __init__(self, func, *args, **kwargs):
        Thread.__init__(self)
        Dispatcher.__init__(self)
        self.func         = func
        self.args         = args
        self.kwargs       = kwargs
        self.data         = None
        self.is_done      = False
        self.err          = None
        self.lock         = Lock()

        core.gear.pool.append(self)
        self.start()

    def run(self):
        try:
            self.data = self.func(*self.args)
        except Exception as err:
            with self.lock:
                self.err = True
        else:
            with self.lock:
                self.is_done = True
        finally:
            waker.wake_up()

    def update(self):
        with self.lock:
            if self.is_done:
                self.process()
        
    def process(self):
        if self.err:
            self.drive(ERROR, self.err)
        else:
            self.drive(DONE, self.data)
        core.gear.pool.remove(self)




